@@ -168,6 +168,8 @@ class GeoArray(object):
             self.flush_cache()  # the cached array is not useful anymore
 
         self._arr = ndarray
+        self._dtype = ndarray.dtype
+        self._shape = ndarray.shape
 
     @property
     def bandnames(self):
@@ -253,13 +255,14 @@ class GeoArray(object):
     @property
     def dtype(self):
         """Get the numpy data type of the associated image array."""
-        if self._dtype:
-            return self._dtype
-        elif self.is_inmem:
+        if self.is_inmem:
             return self.arr.dtype
         else:
-            self.set_gdalDataset_meta()
-            return self._dtype
+            if self._dtype:
+                return self._dtype
+            else:
+                self.set_gdalDataset_meta()
+                return self._dtype
 
     @property
     def geotransform(self):
@@ -507,13 +510,14 @@ class GeoArray(object):
 
     @property
     def metadata(self):
+        # type: () -> GDAL_Metadata
         """Return a DataFrame containing all available metadata (read from file if available).
 
         Use 'metadata[band_index].to_dict()' to get a metadata dictionary for a specific band.
         Use 'metadata.loc[row_name].to_dict()' to get all metadata values of the same key for all bands as dictionary.
         Use 'metadata.loc[row_name, band_index] = value' to set a new value.
 
-        :return:  pandas.DataFrame
+        :return:    instance of GDAL_Metadata
         """
         if self._metadata is not None:
             return self._metadata
@@ -535,7 +539,7 @@ class GeoArray(object):
                              % (self.__class__.__name__, self.__class__.__name__))
         self._metadata = meta
 
-    meta = alias_property('metadata')
+    meta = alias_property('metadata')  # type: GDAL_Metadata
 
     def __getitem__(self, given):
         if isinstance(given, (int, float, slice, np.integer, np.floating)) and self.ndim == 3:
@@ -1297,7 +1301,7 @@ class GeoArray(object):
                 ax = plt.gca()
 
             rows, cols = image2plot.shape[:2]
-            im = ax.imshow(image2plot.astype(int),
+            im = ax.imshow(image2plot,
                            palette,
                            interpolation=interpolation,
                            extent=(0, cols, rows, 0),
@@ -1473,20 +1477,20 @@ class GeoArray(object):
         plt.grid()
         plt.title(title)
 
-        plt.show()
-
         if return_fig:
             return fig
         else:
             plt.show()
 
-    def show_xprofile(self, row, band, xlim=None, ylim=None, figsize=(10, 5), show_nodata=True, return_fig=False):
+    def show_xprofile(self, row, band, xlim=None, ylim=None, title=None, figsize=(10, 5), show_nodata=True,
+                      return_fig=False):
         """Show an x-profile at the given row/band image position.
 
         :param row:             image row number (counts from 0)
         :param band:            image band number (counts from 0)
         :param xlim:            x-axis limits to be used in the plot
         :param ylim:            y-axis limits to be used in the plot
+        :param title:           a custom plot title
         :param figsize:         figure size (tuple)
         :param show_nodata:     whether to show no-data values in the plot
         :param return_fig:      whether to return the figure instead of showing it directly
@@ -1494,17 +1498,19 @@ class GeoArray(object):
         """
         x = range(self.columns)
         y = self[row, :, band]
-        title = f'X-Profile at row {range(self.rows)[row]}, band {range(self.bands)[band]}'
+        title = title or f'X-Profile at row {range(self.rows)[row]}, band {range(self.bands)[band]}'
 
         return self._show_profile(x, y, 'column', 'value', title, xlim, ylim, figsize, show_nodata, return_fig)
 
-    def show_yprofile(self, column, band, xlim=None, ylim=None, figsize=(10, 5), show_nodata=True, return_fig=False):
+    def show_yprofile(self, column, band, xlim=None, ylim=None, title=None, figsize=(10, 5), show_nodata=True,
+                      return_fig=False):
         """Show a y-profile at the given column/band image position.
 
         :param column:          image column number (counts from 0)
         :param band:            image band number (counts from 0)
         :param xlim:            x-axis limits to be used in the plot
         :param ylim:            y-axis limits to be used in the plot
+        :param title:           a custom plot title
         :param figsize:         figure size (tuple)
         :param show_nodata:     whether to show no-data values in the plot
         :param return_fig:      whether to return the figure instead of showing it directly
@@ -1512,17 +1518,19 @@ class GeoArray(object):
         """
         x = range(self.rows)
         y = self[:, column, band]
-        title = f'Y-Profile at column {range(self.columns)[column]}, band {range(self.bands)[band]}'
+        title = title or f'Y-Profile at column {range(self.columns)[column]}, band {range(self.bands)[band]}'
 
         return self._show_profile(x, y, 'row', 'value', title, xlim, ylim, figsize, show_nodata, return_fig)
 
-    def show_zprofile(self, row, column, xlim=None, ylim=None, figsize=(10, 5), show_nodata=True, return_fig=False):
+    def show_zprofile(self, row, column, xlim=None, ylim=None, title=None, figsize=(10, 5), show_nodata=True,
+                      return_fig=False):
         """Show a z-profile at the given row/column image position.
 
         :param row:             image row number (counts from 0)
         :param column:          image column number (counts from 0)
         :param xlim:            x-axis limits to be used in the plot
         :param ylim:            y-axis limits to be used in the plot
+        :param title:           a custom plot title
         :param figsize:         figure size (tuple)
         :param show_nodata:     whether to show no-data values in the plot
         :param return_fig:      whether to return the figure instead of showing it directly
@@ -1540,7 +1548,7 @@ class GeoArray(object):
             x_label = 'band'
 
         y = self[row, column, :]
-        title = f'Z-Profile at row {range(self.rows)[row]}, column {range(self.columns)[column]}'
+        title = title or f'Z-Profile at row {range(self.rows)[row]}, column {range(self.columns)[column]}'
 
         return self._show_profile(x, y, x_label, 'value', title, xlim, ylim, figsize, show_nodata, return_fig)
 
